#pragma once
#include "ffgl/ParamFloat.h"

namespace ffglqs
{

struct ParamsVec
{
	union
	{
		ffgl::ParamFloat* elem[ 4 ];

		struct
		{
			union
			{
				ffgl::ParamFloat* x;
				ffgl::ParamFloat* u;
				ffgl::ParamFloat* red;
				ffgl::ParamFloat* hue;
			};

			union
			{
				ffgl::ParamFloat* y;
				ffgl::ParamFloat* v;
				ffgl::ParamFloat* green;
				ffgl::ParamFloat* sat;
			};

			union
			{
				ffgl::ParamFloat* z;
				ffgl::ParamFloat* s;
				ffgl::ParamFloat* blue;
				ffgl::ParamFloat* bri;
			};

			union
			{
				ffgl::ParamFloat* w;
				ffgl::ParamFloat* t;
				ffgl::ParamFloat* alpha;
			};
		};
	};

	// ------------------------------------------------------------

	inline ffglex::HSBAColor GetHSBColor()
	{
		ffglex::HSBAColor hsb;
		GetHSBColor( hsb );
		return hsb;
	}

	inline void GetHSBColor( ffglex::HSBAColor& hsb )
	{
		hsb.hue   = hue->GetFloatValue();
		hsb.sat   = sat->GetFloatValue();
		hsb.bri   = bri->GetFloatValue();
		hsb.alpha = 1.0f;
	}

	inline ffglex::RGBAColor GetHSBColorAsRGB()
	{
		ffglex::HSBAColor hsb;
		ffglex::RGBAColor rgb;
		GetHSBColor( hsb );
		ffglex::HSBtoRGB( hsb, rgb );
		return rgb;
	}

	inline void GetHSBColorAsRGB( ffglex::RGBAColor& rgb )
	{
		ffglex::HSBAColor hsb;
		GetHSBColor( hsb );
		ffglex::HSBtoRGB( hsb, rgb );
	}

	// ------------------------------------------------------------

	inline ffglex::HSBAColor GetHSBAColor()
	{
		ffglex::HSBAColor hsba;
		GetHSBAColor( hsba );
		return hsba;
	}

	inline void GetHSBAColor( ffglex::HSBAColor& hsba )
	{
		hsba.hue = hue->GetFloatValue();
		hsba.sat = sat->GetFloatValue();
		hsba.bri = bri->GetFloatValue();
		hsba.alpha = alpha->GetFloatValue();
	}

	inline ffglex::RGBAColor GetHSBAColorAsRGBA()
	{
		ffglex::HSBAColor hsba;
		ffglex::RGBAColor rgba;
		GetHSBAColor( hsba );
		ffglex::HSBtoRGB( hsba, rgba );
		return rgba;
	}

	inline void GetHSBAColorAsRGBA( ffglex::RGBAColor& rgba )
	{
		ffglex::HSBAColor hsba;
		GetHSBAColor( hsba );
		ffglex::HSBtoRGB( hsba, rgba );
	}

	// ------------------------------------------------------------

	inline ffglex::RGBAColor GetRGBColor()
	{
		ffglex::RGBAColor rgb;
		GetRGBColor( rgb );
		return rgb;
	}

	inline void GetRGBColor( ffglex::RGBAColor& rgb )
	{
		rgb.r = red->GetFloatValue();
		rgb.g = green->GetFloatValue();
		rgb.b = blue->GetFloatValue();
		rgb.a = 1.0f;
	}

	// ------------------------------------------------------------

	inline ffglex::RGBAColor GetRGBAColor()
	{
		ffglex::RGBAColor rgba;
		GetRGBAColor( rgba );
		return rgba;
	}

	inline void GetRGBAColor( ffglex::RGBAColor& rgba )
	{
		rgba.r = red->GetFloatValue();
		rgba.g = green->GetFloatValue();
		rgba.b = blue->GetFloatValue();
		rgba.a = alpha->GetFloatValue();
	}
};

}// namespace ffglqs
